@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2020, The Monero Project
+// Copyright (c) 2018-2022, The Monero Project
 // 
 // All rights reserved.
 // 
@@ -27,6 +27,7 @@
 // THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <boost/archive/portable_binary_iarchive.hpp>
+#include <boost/filesystem.hpp>
 #include "cryptonote_config.h"
 #include "include_base_utils.h"
 #include "string_tools.h"
@@ -292,12 +293,13 @@ namespace cryptonote
     MINFO("loading rpc payments data from " << state_file_path);
     std::ifstream data;
     data.open(state_file_path, std::ios_base::binary | std::ios_base::in);
+    std::string bytes(std::istream_iterator<char>{data}, std::istream_iterator<char>{});
     if (!data.fail())
     {
       bool loaded = false;
       try
       {
-        binary_archive<false> ar(data);
+        binary_archive<false> ar{epee::strspan<std::uint8_t>(bytes)};
         if (::serialization::serialize(ar, *this))
           if (::serialization::check_stream_state(ar))
             loaded = true;
@@ -305,6 +307,8 @@ namespace cryptonote
       catch (...) {}
       if (!loaded)
       {
+        bytes.clear();
+        bytes.shrink_to_fit();
         try
         {
           boost::archive::portable_binary_iarchive a(data);
